/*
  Sample usage of the RectLayout class
  Copyright (C) 2006 by Javier Arvalo (http://www.iguanademos.com/Jare)

  Uses the GLut library by Mark Kilgard (3.7.6 Win32 version by Nate Robins, patched by some folk at realmtech)
  http://www.xmission.com/~nate/glut.html
  http://www.realmtech.net/opengl/glut.php
*/

#include <stdlib.h>
#include <vector>
#include <gl/glut.h>
#include "RectLayout.h"

struct TRect: public CRectLayout
{
  float r, g, b;

  TRect(float _r, float _g, float _b,
        float _x, float _y, float _w, float _h,
        unsigned _tx = F_SET, unsigned _ty = F_SET, unsigned _tw = F_SET, unsigned _th = F_SET)
     : CRectLayout(_x, _y, _w, _h, _tx, _ty, _tw, _th), r(_r), g(_g), b(_b) { }
};

std::vector<TRect> Rects;

float w, h;

// ------------------------------------
// ------------------------------------
void ReshapeFunc(int width, int height)
{
  w = width;
  h = height;
  glViewport(0, 0, width, height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, width, height, 0, -1.0, 1.0);

  char buf[10000];
  sprintf(buf, "Layout Test: %dx%d", width, height);
  glutSetWindowTitle(buf);
}

// ------------------------------------
// ------------------------------------
void DisplayFunc()
{
  glClearColor(0, 0, .05f*rand()/RAND_MAX, 1);
//  glClearColor(0, 0, 0, 1);
  glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glBlendFunc(GL_ONE, GL_ONE);
  glEnable(GL_BLEND);
  glEnable(GL_TEXTURE_2D);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glColor3f(1, 1, 1);

  for (std::vector<TRect>::iterator i = Rects.begin(); i != Rects.end(); ++i)
  {
    i->Calc(0, 0, w, h);
    glColor3f(i->r, i->g, i->b);
    glBegin(GL_QUADS);
      glVertex2f(i->x,      i->y);
      glVertex2f(i->x,      i->y+i->h);
      glVertex2f(i->x+i->w, i->y+i->h);
      glVertex2f(i->x+i->w, i->y);
    glEnd();
  }

  glutSwapBuffers();
  glFlush();
}

// ------------------------------------
// ------------------------------------
int main(int argc, char **argv)
{
  glutInit(&argc, argv);
  glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE);
  glutInitWindowSize(500, 400);
  glutCreateWindow("Layout Test");
  glutReshapeFunc(ReshapeFunc);
  glutDisplayFunc(DisplayFunc);

  Rects.push_back(TRect(.5f, 0, 0,
        0, 0, 200.f, .25f,
        CRectLayout::F_SET,                             // Anchored to the left side
        CRectLayout::F_END,                             // Anchored to the bottom
        CRectLayout::F_SET,                             // Width is absolute 200
        CRectLayout::F_FACTOR | CRectLayout::F_OTHER)); // Height is 25% of the width (i.e. 125)

  Rects.push_back(TRect(.5f, .5f, 0,
        0, 0, 300.f, .25f,
        CRectLayout::F_END,                             // Anchored to the right side
        CRectLayout::F_END,                             // Anchored to the bottom
        CRectLayout::F_END,                             // Width is canvas's minus 300
        CRectLayout::F_FACTOR | CRectLayout::F_OTHER)); // Height is 25% of the width

  Rects.push_back(TRect(0, .5f, 0,
        .5f, 50.f, .4f, .25f,
        CRectLayout::F_FACTOR,                          // Aligned to the center of the canvas
        CRectLayout::F_SET,                             // Anchored to the 50 below the top
        CRectLayout::F_FACTOR,                          // Width is 40% of the canvas' width
        CRectLayout::F_FACTOR | CRectLayout::F_OTHER)); // Height is 25% of the calculated width

  Rects.push_back(TRect(.5f, 0, .5f,
        10.f, 5.f, .3f, .2f,
        CRectLayout::F_SET,                             // Anchored 10 from the right side
        CRectLayout::F_SET,                             // Anchored 5 from the top
        CRectLayout::F_FACTOR,                          // Width is 30% of the canvas' width
        CRectLayout::F_FACTOR | CRectLayout::F_OTHER)); // Height is 20% of the canvas' height

  Rects.push_back(TRect(0, 0, .5f,
        .5f, .25f, 4.f, .1f,
        CRectLayout::F_FACTOR | CRectLayout::F_CENTER,  // Centered horizontally around the center of the canvas
        CRectLayout::F_FACTOR | CRectLayout::F_CENTER,  // Centered vertically around the 1/4 of the canvas
        CRectLayout::F_FACTOR | CRectLayout::F_OTHER ,  // Width is 400% of the calculated height
        CRectLayout::F_FACTOR));                        // Height is 10% of the canvas' height

  Rects.push_back(TRect(0, .5f, .5f,
        0, 0, 200.f, 100.f,
        CRectLayout::F_END,                             // Anchored to the rightside
        CRectLayout::F_SET,                             // Anchored to the top
        CRectLayout::F_SET,                             // Width is absolute 200
        CRectLayout::F_SET));                           // Height is absolute 100

  glutMainLoop();

  return 0;
}
